% This is the wrapper for fitting exercise 2, the steady-erosion one. 
%
% Greg Balco - Berkeley Geochronology Center - 2018
% Not licensed for reuse or distribution

clear all; close all;

% Get data
d = create_BCO_data();

% Atmospheric pressure
p.h = antatm(2183);

% Surface production rate

P10_St = 33.2; % This is inferred from v3 calculators with CRONUS "Primary" data set
P21_St = P10_St.*4; % Ratio is basically the average of lots of studies...

p.P0 = P21_St;

% Other constants for muon  production

p.fC = 0.296; % Heisinger 2002b Table 2 for Si in SiO2
p.fD = 0.6559; % Same
p.Natoms = 1.003e22; % For Si in SiO2

% Other constants for nucleogenic production
p.l232 = -log(0.5)./1.405e10;
p.l235 = -log(0.5)./7.038e8;
p.l238 = -log(0.5)./4.468e9;

% Other constants
p.rho = 2.25;
p.Lsp = 140;
p.texp = 14.5e6;

% We need to precompute muon stuff, otherwise it takes too long. 
p.predz = logspace(0,4,100);
mc.Natoms = p.Natoms;
mc.sigma0 = 1;
mc.k_neg = 1.*p.fC.*p.fD;
for a = 1:length(p.predz);
    this_mu = P_mu_total_alpha1(p.predz(a),p.h,mc,'yes');
    p.mstub_fast(a) = this_mu.P_fast;
    p.mstub_neg(a) = this_mu.P_neg;
end;
  
% Uncertainty expansions
p.dnfactor = 0.00; % Define uncert on nucleogenic Ne-21, e.g. 20% is 0.2
p.dmfactor = 1; % Increase measurement uncertainty to test...

% Initial guesses

ee0 = 0.2; % m/Myr
tc0 = 150; % Ma;
La0 = 100; % g/cm2
fa0 = 50; % albedo loss percent
fstar0 = 1; % fraction in 10^-3
sigma00 = 0.5./190; % millibarns

x0 = [ee0 tc0 La0 fa0 fstar0 sigma00];
xmin = [0 0 0 0 0 0];
xmax = [1 Inf Inf Inf Inf Inf];

%% Optimize

opts = optimset('fmincon');
opts = optimset(opts,'display','iter','tolFun',1e-7);

[optx,fval] = fmincon(@(x) BCO_objective_steadyerosion(x,d,p,0),x0,[],[],[],[],xmin,xmax,[],opts);

opt.ee = optx(1).*p.rho.*1e-4;
opt.tc = optx(2).*1e6;
opt.La = optx(3);
opt.fa = optx(4).*0.01;
opt.fstar = optx(5).*1e-3;
opt.sigma0 = optx(6).*1e-27;

%%
disp(['E = ' sprintf('%0.2f',optx(1)) ' m/Myr; tc = ' sprintf('%0.1f',optx(2)) ' Ma']);
disp(['La = ' sprintf('%0.2f',optx(3)) '; fa = ' sprintf('%0.3f',opt.fa)]);
disp(['fstar = ' sprintf('%0.4f',opt.fstar) '; sigma0 = ' sprintf('%0.4f',optx(6)) ' millibarns']);


%% Plot results

% Here change nucleogenic Ne-21 uncertainty if you want to test its effect
% on model fit statistic.
%p.dnfactor = 0.25; % Define uncert on nucleogenic Ne-21, e.g., 0.1 is 10%

% Get diagnostics
optd = BCO_objective_steadyerosion(optx,d,p,1);

% Generate model curve

pz = [0:5:50 60:10:140 150:50:550 600:200:6000];

% Spallation

pred.N21sp = exp(-pz./p.Lsp).*(p.P0.*p.Lsp./opt.ee).*(1-exp(-p.texp.*(opt.ee./p.Lsp))) - exp(-pz./opt.La).*(opt.fa.*p.P0.*opt.La./opt.ee).*(1-exp(-p.texp.*(opt.ee./opt.La)));

mc.Natoms = p.Natoms;
mc.sigma0 = opt.sigma0;
mc.k_neg = opt.fstar.*p.fC.*p.fD;

for a = 1:length(pz);
    pred.N21mu(a) = integral(@(t) P_mu_total_alpha1((pz(a)+opt.ee.*t),p.h,mc),0,p.texp,'reltol',1e-3);
end;

%% Plot non-nucleogenic concentrations
dd = d;
dd.N21m = d.N21m - optd.N21nuc;
dd.delN21m = sqrt(d.delN21m.^2 + optd.delN21nuc.^2);
plotaxes = plot_to_show_model(dd);

pred.N21 = pred.N21sp + pred.N21mu;

% Main axis

axes(plotaxes(1));
% Plot nucleogenic
plot(optd.N21nuc,dd.z,'o','markersize',10,'color',[0.75 0.75 0.75]);
% Plot model
plot(pred.N21sp,pz,'k--');
plot(pred.N21mu,pz,'k--');
plot(pred.N21,pz,'k');

axes(plotaxes(2));
% Plot model
plot(pred.N21,pz,'k');

axes(plotaxes(3));
% Plot model
plot(pred.N21,pz,'k');

%% Plot normalized residuals on same figure

set1 = find(d.source == 1 | d.source == 3);
set2 = find(d.source == 2);

set1 = find(d.lab == 1);
set2 = find(d.lab == 2);

% Lower 

axes(plotaxes(4));
for a = 1:length(d.z);
    yy = [d.z(a) d.z(a)];
    xx = [optd.miss(a)-1 optd.miss(a)+1];
    plot(xx,yy,'k'); hold on;
end;

slab = 'v^';
for a = 1:length(d.z);
    if d.source(a) == 1; 
        cc = [0 0 0]; 
    elseif d.source(a) == 2;
        cc = [1 1 1];
    elseif d.source(a) == 3;
        cc = [0.5 0.5 0.5];
    end;
    plot(optd.miss(a),d.z(a),['k' slab(d.lab(a))],'markerfacecolor',cc);
end;

set(gca,'ydir','reverse','ylim',[0 6000]);
set(gca,'xlim',[-4 4]);
plot([0 0],get(gca,'ylim'),'k');
grid on;
xlabel('Normalized residual');
set(gca,'yticklabel',[]);

% Middle 

axes(plotaxes(5)); cla;
for a = 1:length(d.z);
    yy = [d.z(a) d.z(a)];
    xx = [optd.miss(a)-1 optd.miss(a)+1];
    plot(xx,yy,'k'); hold on;
end;

slab = 'v^';
for a = 1:length(d.z);
    if d.source(a) == 1; 
        cc = [0 0 0]; 
    elseif d.source(a) == 2;
        cc = [1 1 1];
    elseif d.source(a) == 3;
        cc = [0.5 0.5 0.5];
    end;
    plot(optd.miss(a),d.z(a),['k' slab(d.lab(a))],'markerfacecolor',cc);
end;

set(gca,'ydir','reverse','ylim',[0 500]);
set(gca,'xlim',[-4 4]);
plot([0 0],get(gca,'ylim'),'k');
set(gca,'xticklabel',[],'yticklabel',[]);
grid on;

% Upper

axes(plotaxes(6)); cla;
for a = 1:length(d.z);
    yy = [d.z(a) d.z(a)];
    xx = [optd.miss(a)-1 optd.miss(a)+1];
    plot(xx,yy,'k'); hold on;
end;

slab = 'v^';
for a = 1:length(d.z);
    if d.source(a) == 1; 
        cc = [0 0 0]; 
    elseif d.source(a) == 2;
        cc = [1 1 1];
    elseif d.source(a) == 3;
        cc = [0.5 0.5 0.5];
    end;
    plot(optd.miss(a),d.z(a),['k' slab(d.lab(a))],'markerfacecolor',cc);
end;

set(gca,'ydir','reverse','ylim',[0 50]);
set(gca,'xlim',[-4 4]);
plot([0 0],get(gca,'ylim'),'k');
set(gca,'xticklabel',[],'yticklabel',[]);
grid on;

%% Plot normalized residuals on different figure

set1 = find(d.source == 1 | d.source == 3);
set2 = find(d.source == 2);

set1 = find(d.lab == 1);
set2 = find(d.lab == 2);

figure;
for a = 1:length(d.z);
    xx = [d.z(a) d.z(a)];
    yy = [optd.miss(a)-1 optd.miss(a)+1];
    plot(xx,yy,'k'); hold on;
end;

slab = 'v^';
for a = 1:length(d.z);
    if d.source(a) == 1; 
        cc = [0 0 0]; 
    elseif d.source(a) == 2;
        cc = [1 1 1];
    elseif d.source(a) == 3;
        cc = [0.5 0.5 0.5];
    end;
    plot(d.z(a),optd.miss(a),['k' slab(d.lab(a))],'markerfacecolor',cc);
end;

set(gca,'xscale','log');
p1 = plot(get(gca,'xlim'),[0 0],'k'); grid on;
xlabel('Depth (g cm^{-2})'); ylabel('Normalized residual');
set(gca,'ylim',[-4 4])

%% Plot non-normalized residuals

miss1 = optd.N21p-d.N21m;
dmiss1 = sqrt(optd.delN21p.^2 + d.delN21m.^2);

figure; 
 
for a = 1:length(d.z);
    xx = [d.z(a) d.z(a)];
    yy = [miss1(a)-dmiss1(a) miss1(a)+dmiss1(a)];
    plot(xx,yy,'k'); hold on;
end;

for a = 1:length(d.z);
    if d.source(a) == 1; 
        cc = [0 0 0]; 
    elseif d.source(a) == 2;
        cc = [1 1 1];
    elseif d.source(a) == 3;
        cc = [0.5 0.5 0.5];
    end;
    plot(d.z(a),miss1(a),['k' slab(d.lab(a))],'markerfacecolor',cc);
end;

plot(get(gca,'xlim'),[0 0],'k'); grid on;
set(gca,'xscale','log');
set(gca,'ylim',[-40e6 40e6]);
xlabel('Depth (g cm^{-2})'); ylabel('Residual (Matoms g^{-1})');


